<!DOCTYPE html>
<html>
    <head>
        <title>Fout: {{ type }}</title>
    </head>
    <body>
        <h1>Fout: <code>{{ type }}</code></h1>

        <dl>
            <dt>HTTP Status code</dt>
            <dd><code>{{ status_code }}</code></dd>

            <dt>Default title</dt>
            <dd><code>{{ default_detail }}</code></dd>

            <dt>Default code</dt>
            <dd><code>{{ default_code }}</code></dd>
        </dl>
    </body>
</html>
